/*******************************************************************************

 WINBINDER - A native Windows binding

 Copyright  2004 Hypervisual - see LICENSE.TXT for details
 Author: Rubem Pechansky (pechansky@hypervisual.com)

 Drawing functions

*******************************************************************************/

//----------------------------------------------------------------- DEPENDENCIES

#include "wb.h"

//-------------------------------------------------------------------- VARIABLES

//----------------------------------------------------------- PRIVATE PROTOTYPES

static HDC GetBitmapHDC(HBITMAP hbm);
static HDC hdcMain = NULL;
static HANDLE DrawStart(HANDLE handle);
static BOOL DrawEnd(HANDLE handle);

//-------------------------------------------------------------- PIXEL FUNCTIONS

BOOL wbSetPixel(HANDLE handle, int xPos, int yPos, COLORREF color)
{
    BOOL bResult;

	if(!DrawStart(handle))
		return FALSE;

	bResult = (SetPixel(hdcMain, xPos, yPos, color) != (COLORREF)-1);

	if(!DrawEnd(handle))
		return FALSE;
	return bResult;
}

COLORREF wbGetPixel(HANDLE handle, int xPos, int yPos)
{
	COLORREF clResult;

	if(!DrawStart(handle))
		return CLR_INVALID;

	clResult = GetPixel(hdcMain, xPos, yPos);

	if(!DrawEnd(handle))
		return CLR_INVALID;
	return clResult;
}

//-------------------------------------------------------------- SHAPE FUNCTIONS

/* Draws a line on the handle. (x0, y0) are the first point coordinates,
  (x1, y1) are the end point coordinates. cl is the line color. */

BOOL wbDrawLine(HANDLE handle, int x0, int y0, int x1, int y1, COLORREF cl, UINT nLineWidth)
{
	HPEN hpen;
	HGDIOBJ hOldObj;

	if(!DrawStart(handle))
		return FALSE;

	hpen = CreatePen(PS_SOLID, nLineWidth, cl);
	hOldObj = SelectObject(hdcMain, hpen);
	MoveToEx(hdcMain, x0, y0, NULL);
	LineTo(hdcMain, x1, y1);
	SelectObject(hdcMain, hOldObj);
	DeleteObject(hpen);

	if(!DrawEnd(handle))
		return FALSE;
	return TRUE;
}

/* Draws a rectangle. xPos, yPos, nWidth and nHeight define the area of the
  rectangle. Set bFilled to TRUE for a filled rectangle, FALSE for a hollow rectangle.
  cl is the fill color of the filled rectangle or the line color of the hollow rectangle.

  TODO: linewidth is not used yet
  */

BOOL wbDrawRect(HANDLE handle, int xPos, int yPos, int nWidth, int nHeight, COLORREF cl, BOOL bFilled, UINT nLineWidth)
{
	RECT rc;
	HBRUSH hbr;
	HGDIOBJ hOldObj;

	if(!DrawStart(handle))
		return FALSE;

	rc.left = xPos;
	rc.top = yPos;
	rc.right = xPos + nWidth;
	rc.bottom = yPos + nHeight;

	if(bFilled) {
		if(cl != CLR_INVALID) {
			hbr = CreateSolidBrush(cl);
			hOldObj = SelectObject(hdcMain, hbr);
			PatBlt(hdcMain, rc.left, rc.top, rc.right - rc.left,	// Faster than FillRect()
			  rc.bottom - rc.top, PATCOPY);
			SelectObject(hdcMain, hOldObj);
			DeleteObject(hbr);
		}
	} else {
		hbr = CreateSolidBrush(cl);
		hOldObj = SelectObject(hdcMain, hbr);
		FrameRect(hdcMain, &rc, hbr);
		SelectObject(hdcMain, hOldObj);
		DeleteObject(hbr);
	}

	if(!DrawEnd(handle))
		return FALSE;
	return TRUE;
}

//------------------------------------------------------------ PRIVATE FUNCTIONS

/* Return a DC for the bitmap pbm. Must be followed by a call to DeleteDC. */

static HDC GetBitmapDC(HBITMAP hbm)
{
	HDC handle, hdcBmp;

	if(!hbm)
		return NULL;
	handle = GetDC(NULL);
	if(!handle)
		return NULL;
	hdcBmp = CreateCompatibleDC(handle);
	if(!hdcBmp) {
		ReleaseDC(NULL, handle);
		return NULL;
	}
	SelectObject(hdcBmp, hbm);
	ReleaseDC(NULL, handle);
	return hdcBmp;
}

// Sets hdcMain

static HANDLE DrawStart(HANDLE handle)
{
	if(!handle)
		return NULL;

	if(wbIsWBObj(handle, FALSE)) {				// Is the handle a PWBOBJ?
		handle = ((PWBOBJ)handle)->handle;
	}
	if(IsWindow((HWND)handle)) {				// Is the handle a window?
		hdcMain = GetDC((HWND)handle);
		if(hdcMain)
			return hdcMain;
	}
	if(GetDeviceCaps(handle, RASTERCAPS)) {		// Is it a DC?
		hdcMain = handle;
		return hdcMain;
	}
	hdcMain = GetBitmapDC((HBITMAP)handle);		// Is it a bitmap?
	if(hdcMain)
		return hdcMain;
	return NULL;								// Unknown
}

static BOOL DrawEnd(HANDLE handle)
{
	if(!handle || !hdcMain)
		return FALSE;

	if(wbIsWBObj(handle, FALSE))				// Is the handle a PWBOBJ?
		return TRUE;

	if(IsWindow((HWND)handle)) {		// Is the handle a window?

		if(!hdcMain)
			return FALSE;
		return ReleaseDC((HWND)handle, hdcMain);

	} else {							// Must be a DC

		if(DeleteDC(hdcMain)) {
			hdcMain = NULL;
			return TRUE;
		} else
			return FALSE;
	}
}

//------------------------------------------------------------------ END OF FILE
